package com.ether;

import com.ether.WebException.WebCode;
import com.ether.user.User;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.InvalidParameterException;
import java.util.Properties;

public final class WebHelper
{
    private WebHelper()
    {
    }

    @NotNull
    public static String getRequestParameter( @NotNull final HttpServletRequest httpServletRequest, @Nullable final String parameterName, @Nullable final String defaultParameterValue, final boolean canBeNull )
            throws InvalidParameterException
    {
        String parameterValue;
        if( null == parameterName )
            parameterValue = defaultParameterValue;

        parameterValue = httpServletRequest.getParameter( parameterName );

        if( parameterValue == null )
            parameterValue = defaultParameterValue;

        if( !canBeNull && parameterValue == null )
            throw new InvalidParameterException( parameterName );

        return parameterValue;
    }

    public static void writeJsonToResponse( @NotNull final HttpServletResponse httpServletResponse, @NotNull final String data )
            throws IOException
    {
        httpServletResponse.setCharacterEncoding( UTF8Charset.getEncoding() );
        httpServletResponse.setContentType( CONTENT_TYPE_TEXT_JSON );

        final PrintWriter writer = httpServletResponse.getWriter();
        writer.write( data );
    }

    public static void displayAjaxError( @NotNull final HttpServletResponse response, @Nullable final String methodName )
            throws WebException
    {
        try
        {
            response.setStatus( HttpServletResponse.SC_NOT_IMPLEMENTED );
            response.setCharacterEncoding( UTF8Charset.getEncoding() );
            response.setContentType( "text/plain;charset=" + UTF8Charset.getEncoding() );
            final ServletOutputStream outputStream = response.getOutputStream();
            outputStream.print( "error.method_not_implemented : " + methodName + " is not implemented." );
            outputStream.flush();
            outputStream.close();
        }
        catch( IOException e )
        {
            throw new WebException( WebCode.IO_EXCEPTION_ERROR_TO_GET_OUTPUTSTREAM, e );
        }
    }

    /**
     * This method returns the user in session if there is one
     *
     * @param request
     * @return
     */
    public static JSONObject getJSONUser( @NotNull final HttpServletRequest request )
    {
        final JSONObject jSONPeople = new JSONObject();

        final User user = (User) request.getSession().getAttribute( "SES_USER" );
        if( null == user )
            return null;

        jSONPeople.put( "name", user.getLastName() );
        jSONPeople.put( "firstName", user.getFirstName() );
        jSONPeople.put( "role", user.getRole() );
        return jSONPeople;
    }

    /**
     * Returns the property writed in the properties file
     *
     * @param request
     * @param property
     * @return
     * @throws WebException
     */
    public static String getProperty( @NotNull final HttpServletRequest request, @NotNull final String property )
            throws WebException
    {
        try
        {
            final String webInfPath = request.getSession().getServletContext().getRealPath( "WEB-INF" );
            return getProperty( webInfPath, property );
        }
        catch( IOException e )
        {
            throw new WebException( WebException.WebCode.ERROR_TO_READ_FILE_PROPERTIES, e );
        }
    }

    /**
     * Returns the property writed in the properties file
     *
     * @param webInfPath
     * @param property
     * @return
     * @throws IOException
     */
    public static String getProperty( @NotNull final String webInfPath, @NotNull final String property )
            throws IOException
    {
        final Properties prop = new Properties();
        prop.load( new FileInputStream( webInfPath + "/classes/" + PROPERTIES_FILE ) );
        return prop.getProperty( property );
    }

    private static final Log LOGGER = LogFactory.getLog( WebHelper.class );

    public static final int STATUS_CODE_SERVICE_EXCEPTION = 500;
    private static final String CONTENT_TYPE_TEXT_JSON = "text/javascript;charset=" + UTF8Charset.getEncoding();
    private static final String PROPERTIES_FILE = "tapas.properties";
}
